const BOOTLOADER_CMD_NOT_SUPPORT = 0x0000;
const BOOTLOADER_CMD_PING = 0x0001;
const BOOTLOADER_CMD_DOWNLOAD_512 = 0x0101;
const BOOTLOADER_CMD_DOWNLOAD_50 = 0x0102;
const BOOTLOADER_CMD_UPLOAD_512 = 0x0201;
const BOOTLOADER_CMD_UPLOAD_50 = 0x0202;

const BOOTLOADER_AREA_DAPP_IDX = 0;
const BOOTLOADER_AREA_HAPP_IDX = 1;
const BOOTLOADER_AREA_DATA_IDX = 2;
const BOOTLOADER_AREA_CONFIG_IDX = 3;
const BOOTLOADER_AREA_SN_HW_IDX = 4;
const BOOTLOADER_AREA_SN_SW_IDX = 5;
const BOOTLOADER_AREA_NUM = 6;

const BOOTLOADER_REQ_RESP_HEAD_LEN = 6;


function bootloader_cmd_get_ping(req_idx, data) {
    var head = new Uint16Array(3);
    head[0] = req_idx;
    head[1] = BOOTLOADER_CMD_PING;
    if (data == null) {
        head[2] = 0;
        return new Uint8Array(head.buffer);
    } else {
        // TODO
        head[2] = data.length;
        var buf = new Uint8Array(6 + data.length);
        buf.set(new Uint8Array(head.buffer), 0);
        buf.set(data, 6);
        return buf;
    }
}

function bootloader_cmd_get_download_512(req_idx, area_idx, pos, data)
{
    var head = new Uint16Array(3);
    head[0] = req_idx;
    head[1] = BOOTLOADER_CMD_DOWNLOAD_512;
    head[2] = 7 + 512;

    var head1 = new Uint8Array(1);
    head1[0] = area_idx;

    var head2 = new Uint16Array(1);
    head2[0] = 512;
    
    var head3 = new Uint32Array(1);
    head3[0] = pos;

    var array = new Uint8Array(6 + 7 + 512);
    array.set(new Uint8Array(head.buffer), 0);
    array.set(head1, 6);
    array.set(new Uint8Array(head2.buffer), 7);
    array.set(new Uint8Array(head3.buffer), 9);
    array.set(data, 13);

    return array;
}

function bootloader_cmd_get_upload_512(req_idx, area_idx, pos)
{
    var head = new Uint16Array(3);
    head[0] = req_idx;
    head[1] = BOOTLOADER_CMD_UPLOAD_512;
    head[2] = 7;

    var head1 = new Uint8Array(1);
    head1[0] = area_idx;

    var head2 = new Uint16Array(1);
    head2[0] = 512;

    var head3 = new Uint32Array(1);
    head3[0] = pos;

    var array = new Uint8Array(6 + 7);
    array.set(new Uint8Array(head.buffer), 0);
    array.set(head1, 6);
    array.set(new Uint8Array(head2.buffer), 7);
    array.set(new Uint8Array(head3.buffer), 9);

    return array;
}

function bootloader_cmd_get_download_50(req_idx, area_idx, pos, data, size)
{
    var head = new Uint16Array(3);
    head[0] = req_idx;
    head[1] = BOOTLOADER_CMD_DOWNLOAD_50;
    head[2] = 7 + 50;

    var head1 = new Uint8Array(1);
    head1[0] = area_idx;

    var head2 = new Uint16Array(1);
    head2[0] = size;
    
    var head3 = new Uint32Array(1);
    head3[0] = pos;

    var array = new Uint8Array(6 + 7 + 50);
    array.set(new Uint8Array(head.buffer), 0);
    array.set(head1, 6);
    array.set(new Uint8Array(head2.buffer), 7);
    array.set(new Uint8Array(head3.buffer), 9);
    array.set(data, 13);

    return array;
}

function bootloader_cmd_get_upload_50(req_idx, area_idx, pos, size)
{
    var head = new Uint16Array(3);
    head[0] = req_idx;
    head[1] = BOOTLOADER_CMD_UPLOAD_50;
    head[2] = 7;

    var head1 = new Uint8Array(1);
    head1[0] = area_idx;

    var head2 = new Uint16Array(1);
    head2[0] = size;
    
    var head3 = new Uint32Array(1);
    head3[0] = pos;

    var array = new Uint8Array(6 + 7);
    array.set(new Uint8Array(head.buffer), 0);
    array.set(head1, 6);
    array.set(new Uint8Array(head2.buffer), 7);
    array.set(new Uint8Array(head3.buffer), 9);

    return array;
}

