let buttonConnectDevice = document.getElementById("connect-device");
let inputVersionShow = document.getElementById("versionShow");

let buttonFwDownloadFile = document.getElementById("fw-download-file");
let buttonFwDownload = document.getElementById("fw-download");
let processFwDownload = document.getElementById("fw-download-process");

let buttonDataDownloadFile = document.getElementById("data-download-file");
let buttonDataDownload = document.getElementById("data-download");
let processDataDownload = document.getElementById("data-download-process");

//let buttonDataUploadFile = document.getElementById("data-upload-file");
let buttonDataUpload = document.getElementById("data-upload");
let processDataUpload = document.getElementById("data-upload-process");

let hidDevice = null;
let hidDeviceHighSpeed = false;   // deafult full speed
let selectFileFwDDownload = null;
let selectFileFwHDownload = null;
let selectFileDataDownload = null;

var timeFun = null;
var taskType = 0;
//var taskTypePending = 0;
//var taskPos = 0;
var taskUpload512Buf = null;
var taskUpload50Buf = null;
var hardwareVersion = null;
var softwareVersion = null;

(async () => {
  initButton();
})();

function taskPing(device, data)
{
  if (taskType) {
    return;
  }

  taskType = BOOTLOADER_CMD_PING;
  
  var array = bootloader_cmd_get_ping(0, data);
  device.sendReport(0, array);
}

function taskDownload50Slice(device, fileData, pos, area_idx)
{
  if (fileData.byteLength <= pos) {
    taskType = 0;
  } else {
    var size = fileData.byteLength - pos;
    if (size > 50) {
      size = 50;
    }
    if (size > (512 - pos % 512)) {
      size = 512 - pos % 512;
    }
    var sliceData = fileData.slice(pos, pos + size);
    var array = bootloader_cmd_get_download_50(0, area_idx, pos, new Uint8Array(sliceData), size);
    device.sendReport(0, array);
    return size;
  }
  return 0;
}

function taskUpload50Slice(device, pos, area_idx) {
  if (taskUpload50Buf.byteLength <= pos) {
    taskType = 0;
  } else {
    var size = taskUpload50Buf.byteLength - pos;
    if (size > 50) {
      size = 50;
    }
    if (size > (512 - pos % 512)) {
      size = 512 - pos % 512;
    }
    var array = bootloader_cmd_get_upload_50(0, area_idx, pos, size);

    console.log(array);
    device.sendReport(0, array);
    return size;
  }
  return 0;
}

function taskDownload50(device, fileData, area_idx)
{
  if (taskType) {
    return;
  }

  taskType = BOOTLOADER_CMD_DOWNLOAD_50;
  taskDownload50Slice(device, fileData, 0, area_idx);
}

function taskDownload512Slice(device, fileData, pos, area_idx)
{
  if (fileData.byteLength <= pos) {
    taskType = 0;
  } else {
    var size = fileData.byteLength - pos;
    if (size > 512) {
      size = 512;
    }
    var sliceData = fileData.slice(pos, pos + size);
    var array = bootloader_cmd_get_download_512(0, area_idx, pos, new Uint8Array(sliceData));
    device.sendReport(0, array);
    return size;
  }
  return 0;
}

function taskDownload512(device, fileData, area_idx)
{
  if (taskType) {
    return;
  }

  taskType = BOOTLOADER_CMD_DOWNLOAD_512;
  taskDownload512Slice(device, fileData, 0, area_idx);
}

function taskUpload512Slice(device, pos, area_idx) {
  if (taskUpload512Buf.byteLength <= pos) {
    taskType = 0;
  } else {
    var size = taskUpload512Buf.byteLength - pos;
    if (size > 512) {
      size = 512;
    }
    var array = bootloader_cmd_get_upload_512(0, area_idx, pos);

    console.log(array);
    device.sendReport(0, array);
    return size;
  }
  return 0;
}

function taskUpload50(device, area_idx, length)
{
  if (taskType) {
    return;
  }

  taskType = BOOTLOADER_CMD_UPLOAD_50;
  taskDownload50AreaIdx = area_idx;
  taskUpload50Buf = new Uint8Array(length);
  taskUpload50Slice(device, 0, area_idx);
}

function taskUpload512(device, area_idx, length)
{
  if (taskType) {
    return;
  }

  taskType = BOOTLOADER_CMD_UPLOAD_512;
  taskDownload512AreaIdx = area_idx;
  taskUpload512Buf = new Uint8Array(length);
  taskUpload512Slice(device, 0, area_idx);
}

function parseRespDownload512(device, data)
{
  const area_idx = data.getUint8(6, true);
  const size = data.getUint16(7, true);
  const pos = data.getUint32(9, true);

  switch (area_idx) {
    case BOOTLOADER_AREA_DAPP_IDX:
      if (taskDownload512Slice(device, selectFileFwDDownload, pos + size, area_idx) == 0) {
        processFwDownload.value = 100 * selectFileFwDDownload.byteLength / (selectFileFwDDownload.byteLength + selectFileFwHDownload.byteLength);
        softwareVersion = null;
        if (selectFileFwHDownload != null) {
            taskDownload512(hidDevice, selectFileFwHDownload, BOOTLOADER_AREA_HAPP_IDX);
        }
      } else {
        processFwDownload.value = pos * (100 * selectFileFwDDownload.byteLength / (selectFileFwDDownload.byteLength + selectFileFwHDownload.byteLength)) / selectFileFwDDownload.byteLength;
      }
      break;
    case BOOTLOADER_AREA_HAPP_IDX:
      if (taskDownload512Slice(device, selectFileFwHDownload, pos + size, area_idx) == 0) {
        processFwDownload.value = 100;
        softwareVersion = null;
      } else {
        processFwDownload.value = 100 * selectFileFwDDownload.byteLength / (selectFileFwDDownload.byteLength + selectFileFwHDownload.byteLength) +
                                    pos * (100 * selectFileFwHDownload.byteLength / (selectFileFwDDownload.byteLength + selectFileFwHDownload.byteLength)) / selectFileFwHDownload.byteLength;
      }
      break;
    case BOOTLOADER_AREA_CONFIG_IDX:
      // TODO
      break;
    case BOOTLOADER_AREA_DATA_IDX:
      if (taskDownload512Slice(device, selectFileDataDownload, pos + size, area_idx) == 0) {
        processDataDownload.value = 100;
      } else {
        processDataDownload.value = pos * 100 / selectFileDataDownload.byteLength;
      }
      break;
  }
}

function parseRespDownload50(device, data)
{
  const area_idx = data.getUint8(6, true);
  const size = data.getUint16(7, true);
  const pos = data.getUint32(9, true);

  switch (area_idx) {
    case BOOTLOADER_AREA_DAPP_IDX:
      if (taskDownload50Slice(device, selectFileFwDDownload, pos + size, area_idx) == 0) {
        processFwDownload.value = 100 * selectFileFwDDownload.byteLength / (selectFileFwDDownload.byteLength + selectFileFwHDownload.byteLength);
        softwareVersion = null;
        if (selectFileFwHDownload != null) {
            taskDownload50(hidDevice, selectFileFwHDownload, BOOTLOADER_AREA_HAPP_IDX);
        }
      } else {
        processFwDownload.value = pos * (100 * selectFileFwDDownload.byteLength / (selectFileFwDDownload.byteLength + selectFileFwHDownload.byteLength)) / selectFileFwDDownload.byteLength;
      }
      break;
    case BOOTLOADER_AREA_HAPP_IDX:
      if (taskDownload50Slice(device, selectFileFwHDownload, pos + size, area_idx) == 0) {
        processFwDownload.value = 100;
        softwareVersion = null;
      } else {
        processFwDownload.value = 100 * selectFileFwDDownload.byteLength / (selectFileFwDDownload.byteLength + selectFileFwHDownload.byteLength) +
                                    pos * (100 * selectFileFwHDownload.byteLength / (selectFileFwDDownload.byteLength + selectFileFwHDownload.byteLength)) / selectFileFwHDownload.byteLength;
      }
      break;
    case BOOTLOADER_AREA_CONFIG_IDX:
      // TODO
      break;
    case BOOTLOADER_AREA_DATA_IDX:
      if (taskDownload50Slice(device, selectFileDataDownload, pos + size, area_idx) == 0) {
        processDataDownload.value = 100;
      } else {
        processDataDownload.value = pos * 100 / selectFileDataDownload.byteLength;
      }
      break;
  }
}

function parseRespUpload512(device, data)
{
  const area_idx = data.getUint8(6, true);
  const size = data.getUint16(7, true);
  const pos = data.getUint32(9, true);

  switch (area_idx) {
    case BOOTLOADER_AREA_SN_HW_IDX:
      console.log('Upload512: BOOTLOADER_AREA_SN_HW_IDX');
      taskUpload512Buf.set(new Uint8Array(data.buffer).slice(13, 13 + 512), pos);
      if (taskUpload512Slice(device, pos + size, area_idx) == 0) {
        hardwareVersion = taskUpload512Buf;
        taskUpload512Buf = null;
        console.log(hardwareVersion);
        console.log('Upload512: Finish');
      }
      break;
    case BOOTLOADER_AREA_SN_SW_IDX:
      console.log('Upload512: BOOTLOADER_AREA_SN_SW_IDX');
      taskUpload512Buf.set(new Uint8Array(data.buffer).slice(13, 13 + 512), pos);
      if (taskUpload512Slice(device, pos + size, area_idx) == 0) {
        softwareVersion = taskUpload512Buf.slice(32, 32 + 24);
        taskUpload512Buf = null;
        console.log(softwareVersion);
        console.log('Upload512: Finish');

        inputVersionShow.value = new TextDecoder().decode(softwareVersion.buffer);
      }
      break;
    case BOOTLOADER_AREA_DATA_IDX:
      console.log('Upload512: BOOTLOADER_AREA_DATA_IDX');
      taskUpload512Buf.set(new Uint8Array(data.buffer).slice(13, 13 + 512), pos);
      if (taskUpload512Slice(device, pos + size, area_idx) == 0) {
        console.log(taskUpload512Buf);
        var testblob = new Blob(taskUpload512Buf);
        console.log(testblob);
        var testblob2 = new Blob([taskUpload512Buf]);
        console.log(testblob2);

        var a = document.createElement("a");
        a.href = window.URL.createObjectURL(new Blob([taskUpload512Buf]));
        a.download = "data.bin";
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);

        processDataUpload.value = 100;
      } else {
        processDataUpload.value = pos * 100 / taskUpload512Buf.byteLength;
      }
      break;
  }
}

function parseRespUpload50(device, data)
{
  const area_idx = data.getUint8(6, true);
  const size = data.getUint16(7, true);
  const pos = data.getUint32(9, true);

  switch (area_idx) {
    case BOOTLOADER_AREA_SN_HW_IDX:
      console.log('Upload50: BOOTLOADER_AREA_SN_HW_IDX, Size:' + size);
      taskUpload50Buf.set(new Uint8Array(data.buffer).slice(13, 13 + size), pos);
      if (taskUpload50Slice(device, pos + size, area_idx) == 0) {
        hardwareVersion = taskUpload50Buf;
        taskUpload50Buf = null;
        console.log(hardwareVersion);
        console.log('Upload50: Finish');
      }
      break;
    case BOOTLOADER_AREA_SN_SW_IDX:
      console.log('Upload50: BOOTLOADER_AREA_SN_SW_IDX, Size:' + size);
      taskUpload50Buf.set(new Uint8Array(data.buffer).slice(13, 13 + size), pos);
      if (taskUpload50Slice(device, pos + size, area_idx) == 0) {
        softwareVersion = taskUpload50Buf.slice(32, 32 + 24);
        taskUpload50Buf = null;
        console.log(softwareVersion);
        console.log('Upload50: Finish');

        inputVersionShow.value = new TextDecoder().decode(softwareVersion.buffer);
      }
      break;
    case BOOTLOADER_AREA_DATA_IDX:
      console.log('Upload50: BOOTLOADER_AREA_DATA_IDX, Size:' + size);
      taskUpload50Buf.set(new Uint8Array(data.buffer).slice(13, 13 + size), pos);
      if (taskUpload50Slice(device, pos + size, area_idx) == 0) {
        console.log(taskUpload50Buf);
        var testblob = new Blob(taskUpload50Buf);
        console.log(testblob);
        var testblob2 = new Blob([taskUpload50Buf]);
        console.log(testblob2);

        var a = document.createElement("a");
        a.href = window.URL.createObjectURL(new Blob([taskUpload50Buf]));
        a.download = "data.bin";
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);

        processDataUpload.value = 100;
      } else {
        processDataUpload.value = pos * 100 / taskUpload50Buf.byteLength;
      }
      break;
  }
}

async function inputReportHandler(event) {
  const { data, device, reportId } = event;

  const resp_idx = data.getUint16(0, true);
  const resp_command = data.getUint16(2, true);
  const resp_data_len = data.getUint16(4, true);

  console.log('resp_idx: ' + resp_idx + 'resp_command: ' + resp_command + 'resp_data_len: ' + resp_data_len);

  switch (resp_command) {
    case BOOTLOADER_CMD_NOT_SUPPORT:
      console.log('Error: BOOTLOADER_CMD_NOT_SUPPORT');
      break;
    case BOOTLOADER_CMD_PING:
      break;
    case BOOTLOADER_CMD_DOWNLOAD_512:
      parseRespDownload512(device, data);
      break;
    case BOOTLOADER_CMD_UPLOAD_512:
      parseRespUpload512(device, data);
      break;
    case BOOTLOADER_CMD_DOWNLOAD_50:
      parseRespDownload50(device, data);
      break;
    case BOOTLOADER_CMD_UPLOAD_50:
      parseRespUpload50(device, data);
      break;
    default:
      console.log('Error: Unknown resp_command: ' + resp_command);
      break;
  }
}

function timeFunHandler() {
  console.log('timeFunHandler');

  if (hardwareVersion == null) {
    if (hidDeviceHighSpeed) {
      taskUpload512(hidDevice, BOOTLOADER_AREA_SN_HW_IDX, 512);
    } else {
      taskUpload50(hidDevice, BOOTLOADER_AREA_SN_HW_IDX, 512);
    }
  } else if (softwareVersion == null) {
    if (hidDeviceHighSpeed) {
      taskUpload512(hidDevice, BOOTLOADER_AREA_SN_SW_IDX, 512);
    } else {
      taskUpload50(hidDevice, BOOTLOADER_AREA_SN_SW_IDX, 512);
    }
  }
}

function disconnectDevice(device) {
  if (timeFun != null) {
    clearInterval(timeFun);
    timeFun = null;
  }

  hardwareVersion = null;
  softwareVersion = null;
  inputVersionShow.value = 'Unconnected';
}

function connectDevice(device) {
  hidDevice = device;
  console.log(hidDevice);

  if (!device.opened) {
    device.open();
  }
  device.oninputreport = (event) => inputReportHandler(event);

  taskType = 0;

  timeFun = setInterval(function () {
    timeFunHandler();
  }, 100);
}

function initButton() {
  buttonConnectDevice.addEventListener("click", async () => {
    if (hidDevice) {
      disconnectDevice(hidDevice);

      buttonFwDownload.disabled = true;
      buttonDataDownload.disabled = true;
      buttonDataUpload.disabled = true;
      buttonConnectDevice.textContent = 'Connect P5 Back Button';
      hidDevice = null;
    } else {
      let devices;
      try {
        devices = await navigator.hid.requestDevice({
          filters: [
            {
              vendorId: 0x6a3a,
              usage: 0x01,
              usagePage: 0x0002
            },
            {
              vendorId: 0x1abe,
              usage: 0x01,
              usagePage: 0x0002
            }
          ],
        });
        connectDevice(devices[0]);
        
        if (selectFileFwDDownload != null) {
          buttonFwDownload.disabled = false;
        }
        if (selectFileFwHDownload != null) {
          buttonFwHDownload.disabled = false;
        }
        if (selectFileDataDownload != null) {
          buttonDataDownload.disabled = false;
        }
        buttonDataUpload.disabled = false;
        buttonConnectDevice.textContent = 'Disconnect P5 Back Button';
      } catch (e) {
        console.log("Connection error: " + e);
      }
    }
  });

  buttonFwDownload.addEventListener("click", async () => {
    if (hidDeviceHighSpeed) {
      taskDownload512(hidDevice, selectFileFwDDownload, BOOTLOADER_AREA_DAPP_IDX);
    } else {
      taskDownload50(hidDevice, selectFileFwDDownload, BOOTLOADER_AREA_DAPP_IDX);
    }
    processFwDownload.value = 0;
  });

  buttonFwDownloadFile.addEventListener("change", async () => {
    selectFileFwDDownload = null;
    buttonFwDownload.disabled = true;

    if (buttonFwDownloadFile.files.length > 0) {
      let file = buttonFwDownloadFile.files[0];
      let reader = new FileReader();
      reader.onload = function () {
        if (reader.result.byteLength > 512) {
          var uint32Head = new Uint32Array(reader.result.slice(0, 8));
          var fwDSize = uint32Head[0];
          var fwHSize = uint32Head[1];
          if (fwDSize && fwHSize && (reader.result.byteLength == (512 + fwDSize + fwHSize))) {
            selectFileFwDDownload = reader.result.slice(512, 512 + fwDSize);
            selectFileFwHDownload = reader.result.slice(512 + fwDSize, 512 + fwDSize + fwHSize);
            if (hidDevice) {
              buttonFwDownload.disabled = false;
            }
          }
        }
      };
      reader.readAsArrayBuffer(file);
    }
  });

  buttonDataDownload.addEventListener("click", async () => {
    if (hidDeviceHighSpeed) {
      taskDownload512(hidDevice, selectFileDataDownload, BOOTLOADER_AREA_DATA_IDX);
    } else {
      taskDownload50(hidDevice, selectFileDataDownload, BOOTLOADER_AREA_DATA_IDX);
    }
    taskDownload512(hidDevice, selectFileDataDownload, BOOTLOADER_AREA_DATA_IDX);
    processDataDownload.value = 0;
  });

  buttonDataDownloadFile.addEventListener("change", async () => {
    selectFileDataDownload = null;
    buttonDataDownload.disabled = true;

    if (buttonDataDownloadFile.files.length > 0) {
      let file = buttonDataDownloadFile.files[0];
      let reader = new FileReader();
      reader.onload = function () {
        if (reader.result.byteLength == 256 * 1024) {
          selectFileDataDownload = reader.result;
          if (hidDevice) {
            buttonDataDownload.disabled = false;
          }
        }
      };
      reader.readAsArrayBuffer(file);
    }
  });

  buttonDataUpload.addEventListener("click", async () => {
    if (hidDeviceHighSpeed) {
      taskUpload512(hidDevice, BOOTLOADER_AREA_DATA_IDX, 256 * 1024);
    } else {
      taskUpload50(hidDevice, BOOTLOADER_AREA_DATA_IDX, 256 * 1024);
    }
    processDataUpload.value = 0;
  });
}

